﻿using System;
using System.Collections.Generic;
using IMSES_Template.DataAccess;
using IMSES_Template.Model;

namespace IMSES_Sample
{
    class Example
    {
        // Please fill in the values below, as theses are needed to run the examples:
        private const string UserName = "";
        private const string Password = "";
        private const ItslearningServers Server = ItslearningServers.Test;
        // Or direct it to your local ImsEs file, i.e. 
        //private const string Server = "https://localhost:446/ImsEnterpriseServicesPort.svc";

        static void Main()
        {
            // In this file two examples are shown, to familiarize yourself with the functions in this template.
            
            // Before the examples below can be excecuted however, you should set up ImsEs on the Itslearning sever 
            // you are going to use, and fill in the username and password in the fields above. Instructions on how 
            // to set-up ImsEs in Itslearning can be found in the official itslearning IMS-ES documentation. There
            // you can also find more documentation on the exact functions in this templeate.

            // Note: The examples below can only be run once, the second time the results will be different, as some
            // of the objects will still be in the trash can of Itslearning.

            try
            {
                // A simple example, to show the basic functionality of the ImsEs Template.
                //SimpleExample();

                // A bit more advanced example, to show the more difficult functionality.
                // (please fill in the SyncId of the main organisation if the site you are testing)
                AdvancedExample("");
            }
            catch (Exception ex)
            {
                var batchEx = ex as ImsEsBatchException;
                if (batchEx != null)
                {
                    foreach (var errorResponse in batchEx.BatchOperationResult.ErrorMessages)
                    {
                        Console.WriteLine("{0} - {1}", errorResponse.Identifier, errorResponse.Message);
                    }
                }
                else
                {
                    Console.WriteLine(ex.ToString());
                }
            }
            
            // Make sure the console doesn't disappear immediately.
            Console.ReadKey();
        }
        
        /// <summary>
        /// A simple example, showing the basics in using the ImsEs Template to communicate with Itslearning.
        /// </summary>
        private static void SimpleExample()
        {
            Console.WriteLine("-start: Simple Example-");

            Console.WriteLine("\n" + "Start with setting up an ImsEs Client.");
            using (var client = new ImsEsClient(UserName, Password, Server) { MessageIdentifier = "SimpleExample"})
            {
                Console.WriteLine("\n" + "Let's start by creating a new User object.");

                var bigBird = new User("big_bird")
                    {
                        FirstName = "Big",          // Required
                        LastName = "Bird",          // Required
                        UserName = "b.bird",        // Required
                        Profile = Profile.Guest,    // Required, but initialized by default to Guest.

                        Address = new Address
                            {
                                AddressLine1 = "123a Sesame Street",
                                AddressLine2 = "(large nest behind 123 Sesame Street)",
                                PostalCode = "10128",
                                City = "New York"
                            }
                    };
                Console.WriteLine(" - Big Bird created!");


                Console.WriteLine("\n" + "Now to add Big Bird to Itslearning.");

                var warning = client.CreateUser(bigBird);
                Console.WriteLine(warning.Status == ResultStatus.Warning ? warning.Message : " - Big Bird added successfully!");


                Console.WriteLine("\n" + "But we forgot to add an email-address to Big Bird.");
                
                bigBird.Email = "big.bird@sesamestreet.com";
                Console.WriteLine(" - Email address added.");


                Console.WriteLine("\n" + "Also make sure that the email is added in Itslearning.");

                warning = client.UpdateUser(bigBird);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Big Bird updated successfully!");


                Console.WriteLine("\n" + "Now we can also check if the update arrived in Itslearning correctly.");

                User testUser;
                warning = client.ReadUser("big_bird", out testUser);
                if (!string.IsNullOrEmpty(warning.Message)) Console.WriteLine(warning.Message);

                Console.WriteLine("\n" + "Big Bird has the following email address in Itslearning:");
                Console.WriteLine(" - " + testUser.Email);


                Console.WriteLine("\n" + "Let's check to which Hierarchies Big Bird has been added.");

                List<Hierarchy> hierarchyList;
                warning = client.ReadHierarchiesFromUser("big_bird", out hierarchyList);
                if (!string.IsNullOrEmpty(warning.Message)) Console.WriteLine(warning.Message);

                Console.WriteLine("\n" + "Big Bird is enrolled in the following Hierarchies:");
                foreach (var hierarchy in hierarchyList)
                {
                    Console.WriteLine(" - " + hierarchy.Title);
                }

                Console.WriteLine("\n" + "It was just the main organisation, as expected.");
                var main = hierarchyList[0];


                Console.WriteLine("\n" + "So create a new Hierarchy, so we can add Big Bird to it.");
                var newHierarchy = new Hierarchy("sesameCreatures")
                    {
                        Title = "Seasmestreet Creatures",       // Required.
                        ParentHierarchyId = main.SyncId         // Required, an Hierarchy always needs a parent.
                    };

                warning = client.CreateHierarchy(newHierarchy);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - New Hierarchy added successfully!");


                Console.WriteLine("\n" + "Now we can add Big Bird to the hierarchy.");
                warning = client.CreateMembership("big_bird", "sesameCreatures", HierarchyRole.Guest);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Big Bird added successfully!");



                Console.WriteLine("\n" + "We can check the Hierarchies of Big Bird again.");

                warning = client.ReadHierarchiesFromUser("big_bird", out hierarchyList);
                if (!string.IsNullOrEmpty(warning.Message)) Console.WriteLine(warning.Message);

                Console.WriteLine("\n" + "Big Bird is enrolled in the following Hierarchies:");
                foreach (var hierarchy in hierarchyList)
                {
                    Console.WriteLine(" - " + hierarchy.Title);
                }


                Console.WriteLine("\n" + "Sadly no animals were allowed at the school, so the Hierarchy was disbanded.");
                warning = client.DeleteHierarchy("sesameCreatures");
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Hierarchy deleted.");


                Console.WriteLine("\n" + "And also Big Bird leaves the school.");
                warning = client.DeleteUser("big_bird");
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Big Bird deleted.");

                Console.WriteLine("\n" + "(in case you need it for the advanced example, the main organisation is \"" + main.SyncId + "\")");
            }

            Console.WriteLine("\n" + "-end-");
        }

        /// <summary>
        /// A bit more advanced example, showing somewhat more diffucult functions from the ImsEs Template.
        /// </summary>
        /// <param name="mainOrganisation"></param>
        private static void AdvancedExample(string mainOrganisation)
        {
            Console.WriteLine("-start: Advanced Example-");

            using (var client = new ImsEsClient(UserName, Password, Server) { MessageIdentifier = "AdvancedExample" })
            {
                Console.Write("\n" + "With ImsEs you can create Hierarchies, like shown in the simple example. ");
                Console.WriteLine("But you can also create new organisations, which works in almost the same way.");

                var newOrganisation = new Hierarchy("occult666", true)
                    {
                        Title = "Occult Department",
                        ParentHierarchyId = mainOrganisation,
                    };

                var warning = client.CreateHierarchy(newOrganisation);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - New organisation created successfully.");

                
                Console.WriteLine("\n" + "In this new organisation we will create a new course, Magic 101.");


                var newHierarchy = new Hierarchy("m101")
                    {
                        Title = "Magic 101 group",
                        ParentHierarchyId = "occult666"
                    };
                
                var newCourse = new Course("m101")
                    {
                        Title = "Magic 101",
                        Code = "M101",
                        LinkedHierarchy = newHierarchy
                    };

                warning = client.CreateCourse(newCourse);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - New course created successfully.");


                Console.Write("\n" + "As you might have seen from the object definition, a course is actually linked to ");
                Console.Write("an hierarchy. When you create a Course, you create a Hierarchy, with some extra information, ");
                Console.WriteLine("which Itslearning then converts to a new course (plus the niew hierarchy of course).");


                Console.WriteLine("\n" + "Drawn to the school by this new Course, four new students apply.");

                var user1 = new User("pHarry")
                    {
                        FirstName = "Harry",
                        LastName = "Potter",
                        UserName = "h.potter",
                        Profile = Profile.Pupil
                    };
                Console.WriteLine(" - Harry Potter");

                var user2 = new User("wMerlin")
                {
                    FirstName = "Merlin",
                    LastName = "the Wizard",
                    UserName = "merlin",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Merlin");

                var user3 = new User("gGandalf")
                {
                    FirstName = "Gandalf",
                    LastName = "the Grey",
                    UserName = "g.grey",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Gandalf the Grey");

                var user4 = new User("hCatweazle")
                {
                    FirstName = "Catweazle",
                    LastName = "the Hermit",
                    UserName = "catweazle",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Catweazle");

                var warnings = client.CreateUsers(new[] {user1, user2, user3, user4});
                if (warnings.WarningMessages != null)
                {
                    foreach (var warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine("\n" + "Four Wizards created successfully.");


                Console.WriteLine("\n" + "The wizards immediately apply to the course.");
                var membershipList = new []
                    {
                        new MembershipPair
                            {
                                UserSyncKey = "pHarry",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            },
                        new MembershipPair
                            {
                                UserSyncKey = "wMerlin",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            },
                        new MembershipPair
                            {
                                UserSyncKey = "gGandalf",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            },
                        new MembershipPair
                            {
                                UserSyncKey = "hCatweazle",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            }
                    };

                warnings = client.CreateMemberships(membershipList);
                if (warnings.WarningMessages != null)
                {
                    foreach (var warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine("\n" + "Four Wizards added to the course.");


                Console.WriteLine("\n" + "Check if they were added correctly:");
                
                List<User> userList;
                client.ReadUsersFromHierarchy("m101", out userList);
                Console.WriteLine("The following Users are enrolled in Magic 101:");
                foreach (var user in userList)
                {
                    Console.WriteLine(" - " + user.FirstName + " " + user.LastName);
                }

                Console.Write("\n" + "As you can see, also the adminstartor(s) of the main organisation are listed here,");
                Console.WriteLine(" they are automatically added to any hierarchy or Course that is added to the organisation they are administrator of.");
                
                
                Console.WriteLine("\n" + "Due to his old age, Gandalf was chosen to be teacher of the Course.");

                warning = client.UpdateMembership("gGandalf", "m101", HierarchyRole.Teacher);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Gandalf successful added as Teacher.");

                
                Console.WriteLine("\n" + "Harry Potter wants to be ingocnito, and therefore requests a different SyncKey.");

                warning = client.ChangeUserKey("pHarry", "hp347");
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Harry Potters UserKey changed.");

                Console.WriteLine("\n" + "Now to check if his old key has indeed been removed:");
                Console.WriteLine("(this will generate an exception, as expected.)");
                
                try
                {
                    User tempUser;
                    client.ReadUser("pHarry", out tempUser);
                }
                catch (Exception ex)
                {
                  Console.WriteLine("\n" + ex);
                }

                
                Console.WriteLine("\n" + "To illustrate the difference between an Update and a Replace, Merlin needs some more information added.");
                
                user2.Phone = "1234566789";
                client.UpdateUser(user2);
                
                Console.WriteLine("\n" + "Before the Update:");
                Console.WriteLine(" - Email = " + user2.Email);
                Console.WriteLine(" - Phone = " + user2.Phone);

                user2.Phone = null;
                user2.Email = "merling@avalon.co.uk";
                client.UpdateUser(user2);
                client.ReadUser("wMerlin", out user2);


                Console.WriteLine("\n" + "After the Update:");
                Console.WriteLine(" - Email = " + user2.Email);
                Console.WriteLine(" - Phone = " + user2.Phone);

                user2.Email = "merlin@avalon.co.uk";
                client.ReplaceUser(user2);
                client.ReadUser("wMerlin", out user2);

                Console.WriteLine("\n" + "After the Replace:");
                Console.WriteLine(" - Email = " + user2.Email);
                Console.WriteLine(" - Phone = " + user2.Phone);

                
                Console.WriteLine("\n" + "Because the wizards do not want to have anything to with the school, except their course, they change their organisation.");

                membershipList = new[]
                    {
                        new MembershipPair { UserSyncKey = "hp347", HierarchySyncId = "occult666" },
                        new MembershipPair { UserSyncKey = "wMerlin", HierarchySyncId = "occult666" },
                        new MembershipPair { UserSyncKey = "gGandalf", HierarchySyncId = "occult666" },
                        new MembershipPair { UserSyncKey = "hCatweazle", HierarchySyncId = "occult666" }
                    };

                warnings = client.SetHomeOrganisations(membershipList);
                if (warnings.WarningMessages != null)
                {
                    foreach (var warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }


                Console.WriteLine("A few days before the start of the semester, the school finds out they made a mistake.");
                Console.WriteLine("It seems the course was actually called Arabic 101, but due to some bad handwriting it was entered incorrectly.");

                newCourse.Title = "Arabic 101";
                newCourse.Code = "A101";

                warning = client.UpdateCourse(newCourse);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : "- Course renamed correctly.");

                
                Console.WriteLine("\n" + "When the four wizards hear about the change, they all leave the school.");

                warnings = client.DeleteUsers(new[] { "hp347", "wMerlin", "gGandalf", "hCatweazle" });
                if (warnings.WarningMessages != null)
                {
                    foreach (var warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine(" - Four wizards deleted successfully.");


                Console.WriteLine("\n" + "With the students gone, the school decides to also delete the course.");
                Console.Write("But here it gets tricky. Through ImsEs you can only delete the linked Hierarchy, ");
                Console.Write("never the course itself. So this will have to be done manually, through the Itslearning UI.");
                Console.WriteLine();
                Console.Write("If you do delete the hierarchy, the Course will remain in Itslearning, but then you will have ");
                Console.WriteLine("no way to change it through ImsEs anymore.");

                client.DeleteHierarchy("m101");
            }

            Console.WriteLine("\n" + "-end-");
        }
        
    }
}
