﻿using System.ComponentModel.DataAnnotations;

namespace IMSES_Template.Model
{
    public abstract class BaseGroup : BaseEntity
    {
        /// <summary>
        /// The SyncId of the Hierarchy (also known as Synchonisation ID).
        /// </summary>
        [Required]
        public string SyncId { get; protected set; }

        /// <summary>
        /// The Title of the Hierarchy.
        /// </summary>
        [Required]
        public string Title { get; set; }

        /// <summary>
        /// Description of the Hierarchy (only in ImsEs)
        /// </summary>
        public string Description { get; set; }

        /// <summary>
        /// The Hierarchy's parent (one level higher). 
        /// </summary>
        [Required]
        public string ParentHierarchyId { get; set; }

        /// <summary>
        /// The Grouptype of this Hierarchy.
        /// </summary>
        [Required]
        public GroupType GroupType { get; internal set; }

        /// <summary>
        /// True/False if the Hierarchy is visible in searches (for example when sending a message).
        /// </summary>
        public bool VisibleInSearch { get; set; }


        /// <summary>
        /// Checks if the object is valid, i.e. that all properties marked with [Required] are actually initialized and filled, 
        /// and throws an exception with the porperties that aren't.
        /// </summary>
        public void CheckRequired()
        {
            CheckRequired(SyncId);
        }

        /// <summary>
        /// Checks if all required fields are filled in, and checks if certain properties are not too long. It also checks the Attached Course, if set.
        /// </summary>
        /// <returns>Either the properties that are required and not filled in, or an error about the length of an property.</returns>
        public void CheckValid()
        {
            CheckSingleValid();
        }

        /// <summary>
        /// Checks if all required fields are filled in, and checks if certain properties are not too long.
        /// </summary>
        /// <returns>Either the properties that are required and not filled in, or an error about the length of an property.</returns>
        internal void CheckSingleValid()
        {
            if (SyncId.Length > 255)
                throw new ValidationException(string.Format("({0}) SyncId is too long, max length is 255 characters.", SyncId));

            if (Title != null && Title.Length > 64)
                throw new ValidationException(string.Format("({0}) Title is too long, max length is 64 characters.", SyncId));
        }

    }
}
