﻿using System;
using System.Collections.Generic;
using IMSES_Template.ImsEnterpriseServicePort;
using IMSES_Template.Model;

namespace IMSES_Template.Utils
{
    public static class ConvertEnums
    {
        private const string Meta = "meta-";


        /// <summary>
        /// A dictionary to handle the mapping between ImsEs InstitutionRoles and Itslearning Profiles.
        /// </summary>
        private static readonly Dictionary<InstitutionRoleDTypeInstitutionRoleType, Profile> UserRoleToProfileMap = new Dictionary<InstitutionRoleDTypeInstitutionRoleType, Profile>
            {
                {InstitutionRoleDTypeInstitutionRoleType.Learner,       Profile.SystemAdministrator},
                {InstitutionRoleDTypeInstitutionRoleType.Administrator, Profile.Administrator},
                {InstitutionRoleDTypeInstitutionRoleType.Staff,         Profile.Staff},
                {InstitutionRoleDTypeInstitutionRoleType.Student,       Profile.Pupil},
                {InstitutionRoleDTypeInstitutionRoleType.Guest,         Profile.Guest}
            };

        /// <summary>
        /// A dictionary to handle the mapping between Itslearning Profiles and ImsEs InstitutionRoles.
        /// </summary>
        private static readonly Dictionary<Profile, InstitutionRoleDTypeInstitutionRoleType> ProfileToUserRoleMap = new Dictionary<Profile, InstitutionRoleDTypeInstitutionRoleType>
            {
                {Profile.SystemAdministrator,   InstitutionRoleDTypeInstitutionRoleType.Learner},
                {Profile.Administrator,         InstitutionRoleDTypeInstitutionRoleType.Administrator},
                {Profile.Staff,                 InstitutionRoleDTypeInstitutionRoleType.Staff},
                {Profile.Pupil,                 InstitutionRoleDTypeInstitutionRoleType.Student},
                {Profile.Guest,                 InstitutionRoleDTypeInstitutionRoleType.Guest}
            };

        /// <summary>
        /// A dictionary to handle the mapping between ImeEs GroupRoles and Itslearning Roles.
        /// </summary>
        private static readonly Dictionary<RoleDTypeRoleType, HierarchyRole> RoleDTypeToRoleMap = new Dictionary<RoleDTypeRoleType, HierarchyRole>
            {
                {RoleDTypeRoleType.Learner,             HierarchyRole.Pupil},
                {RoleDTypeRoleType.Instructor,          HierarchyRole.Teacher},
                {RoleDTypeRoleType.ContentDeveloper,    HierarchyRole.Guest},
                {RoleDTypeRoleType.Member,              HierarchyRole.Guest},
                {RoleDTypeRoleType.Manager,             HierarchyRole.Guest},
                {RoleDTypeRoleType.Mentor,              HierarchyRole.Mentor},
                {RoleDTypeRoleType.Administrator,       HierarchyRole.Administrator},
                {RoleDTypeRoleType.TeachingAssistant,   HierarchyRole.Guest},
                {RoleDTypeRoleType.Item01,              HierarchyRole.Pupil}, // same as Learner
                {RoleDTypeRoleType.Item02,              HierarchyRole.Guest}, // same as Instructor
                {RoleDTypeRoleType.Item03,              HierarchyRole.Guest}, // same as ContentDeveloper
                {RoleDTypeRoleType.Item04,              HierarchyRole.Guest}, // same as Member
                {RoleDTypeRoleType.Item05,              HierarchyRole.Guest}, // same as Manager
                {RoleDTypeRoleType.Item06,              HierarchyRole.Mentor}, // same as Mentor
                {RoleDTypeRoleType.Item07,              HierarchyRole.Administrator}, // same as Administrator
                {RoleDTypeRoleType.Item08,              HierarchyRole.Guest} // same as TeachingAssistant
            };

        /// <summary>
        /// A dictionary to handle the mapping between ImeEs GroupRoles and Itslearning Roles.
        /// </summary>
        private static readonly Dictionary<HierarchyRole, RoleDTypeRoleType> RoleToRoleDTypeMap = new Dictionary<HierarchyRole, RoleDTypeRoleType>
            {
                {HierarchyRole.Guest,           RoleDTypeRoleType.Member},
                {HierarchyRole.Pupil,           RoleDTypeRoleType.Learner},
                {HierarchyRole.Mentor,          RoleDTypeRoleType.Mentor},
                {HierarchyRole.Teacher,         RoleDTypeRoleType.Instructor},
                {HierarchyRole.Administrator,   RoleDTypeRoleType.Administrator},
            };

        
        /// <summary>
        /// Converts an ImsEs InstitutionRole to an Itslearning Profile.
        /// </summary>
        /// <param name="role">An ImsEs InstitutionRoleDTypeInstitutionRoleType.</param>
        /// <returns>An Itslearning Profile</returns>
        public static Profile ToProfile(this InstitutionRoleDTypeInstitutionRoleType role)
        {
            return UserRoleToProfileMap[role];
        }

        /// <summary>
        /// Converts an Itslearning Profile to an ImsEs InstitutionRole.
        /// </summary>
        /// <param name="profile">An Itslearning Profile</param>
        /// <returns>An ImsEs InstitutionRoleDTypeInstitutionRoleType.</returns>
        public static InstitutionRoleDTypeInstitutionRoleType ToInstitutionRole(this Profile profile)
        {
            return ProfileToUserRoleMap[profile];
        }


        /// <summary>
        /// Converts an ImsEs Role to an Itslearning HierarchyRole.
        /// </summary>
        /// <param name="role">An ImsEs Role.</param>
        /// <returns>An Itslearning HierarchyRole.</returns>
        public static HierarchyRole ToHierarchyRole(this RoleDTypeRoleType role)
        {
            return RoleDTypeToRoleMap[role];
        }

        /// <summary>
        /// Converts an Itslearning HierarchyRole to an ImsEs Role.
        /// </summary>
        /// <param name="role">An Itslearning HierarchyRole.</param>
        /// <returns>An ImsEs Role.</returns>
        public static RoleDTypeRoleType ToImsEsRole(this HierarchyRole role)
        {
            return RoleToRoleDTypeMap[role];
        }

        /// <summary>
        /// Convert a String to a NameType enum.
        /// </summary>
        /// <param name="nameType">The string.</param>
        /// <returns>A NameType enum.</returns>
        public static NameFieldType ToNameType(this string nameType)
        {
            try
            {
                return (NameFieldType)Enum.Parse(typeof (NameFieldType), nameType);
            }
            catch (Exception)
            {
                return NameFieldType.Undefined;
            }
        }

        
        /// <summary>
        /// Converts a String to an ExtensionType enum.
        /// </summary>
        /// <param name="extensionType">The string.</param>
        /// <returns>An ExtensionType enum.</returns>
        public static ExtensionFieldType ToExtensionType(this string extensionType)
        {
            // First check for MetaData
            if (extensionType.StartsWith(Meta, StringComparison.InvariantCultureIgnoreCase))
            {
                return ExtensionFieldType.MetaData;
            }

            try
            {
                return (ExtensionFieldType)Enum.Parse(typeof(ExtensionFieldType), extensionType);
            }
            catch (Exception)
            {
                return ExtensionFieldType.Undefined;
            }
        }

        
        /// <summary>
        /// Converts a String to a CourseExtType enum.
        /// </summary>
        /// <param name="courseExtType">The string.</param>
        /// <returns>A CourseExtType enum.</returns>
        public static CourseExtFieldType ToCourseExtType(this string courseExtType)
        {
            try
            {
                return (CourseExtFieldType)Enum.Parse(typeof(CourseExtFieldType), courseExtType);
            }
            catch (Exception)
            {
                return CourseExtFieldType.Undefined;
            }
        }


        /// <summary>
        /// Converts a String to a BoolType enum.
        /// </summary>
        /// <param name="boolType">The String.</param>
        /// <returns>A BoolType enum.</returns>
        public static BoolFieldType ToBoolType(this string boolType)
        {
            try
            {
                return (BoolFieldType)Enum.Parse(typeof(BoolFieldType), boolType);
            }
            catch (Exception)
            {
                return BoolFieldType.Undefined;
            }
        }

        /// <summary>
        /// Converts a byte to the GroupType enum.
        /// </summary>
        /// <param name="groupType">The byte to be converted.</param>
        /// <returns>An Grouptype enum.</returns>
        public static GroupType ToGroupType(this string groupType)
        {
            try
            {
                return (GroupType) byte.Parse(groupType);
            }
            catch (Exception)
            {
                return GroupType.Hierarchy;
            }
        }


    }
}
